
/*****************************************************************************
 *                                                                           *
 * This FILE mx12p_internal.h                                                *
 * is part of the MX12P library.  It is based on code copyright 1996 and     *
 * 1998 by Adimec Advanced Image Systems B.V.  Unix conversion copyright     *
 * 1999 and 2000 by Gregory L. Kreider.  All rights reserved.  See           *
 * mx12p_lib.c for distribution terms.                                       *
 *                                                                           *
 * The code in this file contains header comments used internally by         *
 * mx12p_lib.c.  More comments can be found in libraries.doc                 *
 *                                                                           *
 *****************************************************************************/


#ifndef _MX12P_INT_LIB
#define _MX12P_INT_LIB 1

/****************************************************************************
 *
 * Register at 0x20
 *   bit 0 (0x01)      I0   integration time selection
 *   bit 1 (0x02)      I1   
 *   bit 2 (0x04)      I2   
 *   bit 3 (0x08)      I3   
 *   bit 4 (0x10)      I4   
 *   bit 5 (0x20)      0 = Balanced signal output ; 1 = TTL output
 *   bit 6 (0x40)      0 = 60 HZ ; 1 = 50 Hz
 *   bit 7 (0x80)      option 1 (not currently used)
 *
 * Register at 0x21
 *   bit 0 (0x01)      0 = Sync OFF ; 1 = Sync ON
 *   bit 1 (0x02)      0 = Mirrored read-out ; 1 = Normal readout
 *   bit 2 (0x04)      M0   mode select (00 = control, 01 = gated trigger,
 *   bit 3 (0x08)      M1                10 = trigger single, 11 = cont)
 *   bit 4 (0x10)      0 = 20 MHz pixel clock ; 1 = 40 MHz     (PCSN)
 *   bit 5 (0x20)      0 = progressive read-out ; 1 = interlaced
 *   bit 6 (0x40)      option 2 (not currently used)
 *   bit 7 (0x80)      0 = FT12 sensor ; 1 = FT18
 *
 * Register at 0x22
 *   bit 0 (0x01)      G0   video gain select
 *   bit 1 (0x02)      G1   
 *   bit 2 (0x04)      G2   
 *   bit 3 (0x08)      G3   
 *   bit 4 (0x10)      G4   
 *   bit 5 (0x20)      0 = Coarse black shift ON ; 1 = Coarse shift OFF
 *   bit 6 (0x40)      0 = Gamma correction OFF ; 1 = Gamma correction ON
 *   bit 7 (0x80)      0 = Fine black shift ON ; 1 = Fine shift OFF
 *
 * 8-bit DAC at 0x48
 *   When Fine black shift OFF, shift level = (DAC/255)*100%
 *                         ON,                (DAC/255)*5%
 *
 ****************************************************************************/

/* Camera-specific constants, for internal use */

/* bit masks */
#define BIT_SEQ_SET     0x20u
#define BIT_GAM_SET     0x40u
#define BIT_MIR_SET     0x02u
#define BIT_SYN_SET     0x01u
#define BIT_MOD_VAL     0x0Cu
#define BIT_BLK_SET     0x20u
#define BIT_FIN_SET     0x80u
#define BIT_GAI_VAL     0x1Fu
#define BIT_INT_VAL     0x1Fu
#define BIT_STD_SET     0x40u
#define BIT_TTL_SET     0x20u
#define BIT_OP1_SET     0x80u
#define BIT_PCSN        0x10u
#define BIT_OP2_SET     0x40u
#define BIT_SES         0x80u

/* for setting polarity high (1=high active; 0=low active) */
#define POL_STD_SET         1
#define POL_TTL_SET         1
#define POL_SEQ_SET         1
#define POL_GAM_SET         1
#define POL_MIR_SET         0
#define POL_SYN_SET         1
#define POL_BLK_SET         0
#define POL_FIN_SET         0
#define POL_OP1_SET         1
#define POL_OP2_SET         1
#define POL_SES_SET         1
#define POL_PCSN_SET        1

/* show registers for each setting */
#define SHD_SEQ_SET     io_reg_21
#define SHD_GAM_SET     io_reg_22
#define SHD_MIR_SET     io_reg_21
#define SHD_SYN_SET     io_reg_21
#define SHD_MOD_VAL     io_reg_21
#define SHD_BLK_SET     io_reg_22
#define SHD_FIN_SET     io_reg_22
#define SHD_GAI_VAL     io_reg_22
#define SHD_INT_VAL     io_reg_20
#define SHD_TTL_SET     io_reg_20
#define SHD_STD_SET     io_reg_20
#define SHD_BLK_VAL     dac_reg_48
#define SHD_OP1_SET     io_reg_20
#define SHD_OP2_SET     io_reg_21
#define SHD_SES_SET     io_reg_21
#define SHD_PCSN_SET    io_reg_21

/* macros for setting various flags */
#if POL_OP1_SET == 1
#define REG_OP1_SET(reg,set) if (set) reg|=BIT_OP1_SET; else reg&=~BIT_OP1_SET
#else
#define REG_OP1_SET(reg,set) if (set) reg&=~BIT_OP1_SET; else reg|=BIT_OP1_SET
#endif

#if POL_OP2_SET == 1
#define REG_OP2_SET(reg,set) if (set) reg|=BIT_OP2_SET; else reg&=~BIT_OP2_SET
#else
#define REG_OP2_SET(reg,set) if (set) reg&=~BIT_OP2_SET; else reg|=BIT_OP2_SET
#endif

#if POL_SEQ_SET == 1
#define REG_SEQ_SET(reg,set) if (set) reg|=BIT_SEQ_SET; else reg&=~BIT_SEQ_SET
#else
#define REG_SEQ_SET(reg,set) if (set) reg&=~BIT_SEQ_SET; else reg|=BIT_SEQ_SET
#endif

#if POL_GAM_SET == 1
#define REG_GAM_SET(reg,set) if (set) reg|=BIT_GAM_SET; else reg&=~BIT_GAM_SET
#else
#define REG_GAM_SET(reg,set) if (set) reg&=~BIT_GAM_SET; else reg|=BIT_GAM_SET
#endif

#if POL_MIR_SET == 1
#define REG_MIR_SET(reg,set) if (set) reg|=BIT_MIR_SET; else reg&=~BIT_MIR_SET
#else
#define REG_MIR_SET(reg,set) if (set) reg&=~BIT_MIR_SET; else reg|=BIT_MIR_SET
#endif

#if POL_SYN_SET == 1
#define REG_SYN_SET(reg,set) if (set) reg|=BIT_SYN_SET; else reg&=~BIT_SYN_SET
#else
#define REG_SYN_SET(reg,set) if (set) reg&=~BIT_SYN_SET; else reg|=BIT_SYN_SET
#endif

#if POL_BLK_SET == 1
#define REG_BLK_SET(reg,set) if (set) reg|=BIT_BLK_SET; else reg&=~BIT_BLK_SET
#else
#define REG_BLK_SET(reg,set) if (set) reg&=~BIT_BLK_SET; else reg|=BIT_BLK_SET
#endif

#if POL_FIN_SET == 1
#define REG_FIN_SET(reg,set) if (set) reg|=BIT_FIN_SET; else reg&=~BIT_FIN_SET
#else
#define REG_FIN_SET(reg,set) if (set) reg&=~BIT_FIN_SET; else reg|=BIT_FIN_SET
#endif

#if POL_STD_SET == 1
#define REG_STD_SET(reg,set) if (set) reg|=BIT_STD_SET; else reg&=~BIT_STD_SET
#else
#define REG_STD_SET(reg,set) if (set) reg&=~BIT_STD_SET; else reg|=BIT_STD_SET
#endif

#if POL_TTL_SET == 1
#define REG_TTL_SET(reg,set) if (set) reg|=BIT_TTL_SET; else reg&=~BIT_TTL_SET
#else
#define REG_TTL_SET(reg,set) if (set) reg&=~BIT_TTL_SET; else reg|=BIT_TTL_SET
#endif

#if POL_SES_SET == 1
#define REG_SES_SET(reg,set) if (set) reg|=BIT_SES; else reg&=~BIT_SES
#else
#define REG_SES_SET(reg,set) if (set) reg&=~BIT_SES; else reg|=BIT_SES
#endif

#if POL_PCSN_SET == 1
#define REG_PCSN_SET(reg,set) if (set) reg|=BIT_PCSN; else reg&=~BIT_PCSN
#else
#define REG_PCSN_SET(reg,set) if (set) reg&=~BIT_PCSN; else reg|=BIT_PCSN
#endif

#if POL_OP1_SET == 1
#define VAL_OP1_SET(reg)  ((reg & BIT_OP1_SET) > 0)
#else
#define VAL_OP1_SET(reg)  ((reg & BIT_OP1_SET) == 0)
#endif

#if POL_OP2_SET == 1
#define VAL_OP2_SET(reg)  ((reg & BIT_OP2_SET) > 0)
#else
#define VAL_OP2_SET(reg)  ((reg & BIT_OP2_SET) == 0)
#endif

#if POL_SEQ_SET == 1
#define VAL_SEQ_SET(reg)  ((reg & BIT_SEQ_SET) > 0)
#else
#define VAL_SEQ_SET(reg)  ((reg & BIT_SEQ_SET) == 0)
#endif

#if POL_GAM_SET == 1
#define VAL_GAM_SET(reg)  ((reg & BIT_GAM_SET) > 0)
#else
#define VAL_GAM_SET(reg)  ((reg & BIT_GAM_SET) == 0)
#endif

#if POL_OP1_SET == 1
#define VAL_OP1_SET(reg)  ((reg & BIT_OP1_SET) > 0)
#else
#define VAL_OP1_SET(reg)  ((reg & BIT_OP1_SET) == 0)
#endif

#if POL_MIR_SET == 1
#define VAL_MIR_SET(reg)  ((reg & BIT_MIR_SET) > 0)
#else
#define VAL_MIR_SET(reg)  ((reg & BIT_MIR_SET) == 0)
#endif

#if POL_SYN_SET == 1
#define VAL_SYN_SET(reg)  ((reg & BIT_SYN_SET) > 0)
#else
#define VAL_SYN_SET(reg)  ((reg & BIT_SYN_SET) == 0)
#endif

#if POL_BLK_SET == 1
#define VAL_BLK_SET(reg)  ((reg & BIT_BLK_SET) > 0)
#else
#define VAL_BLK_SET(reg)  ((reg & BIT_BLK_SET) == 0)
#endif

#if POL_FIN_SET == 1
#define VAL_FIN_SET(reg)  ((reg & BIT_FIN_SET) > 0)
#else
#define VAL_FIN_SET(reg)  ((reg & BIT_FIN_SET) == 0)
#endif

#if POL_STD_SET == 1
#define VAL_STD_SET(reg)  ((reg & BIT_STD_SET) > 0)
#else
#define VAL_STD_SET(reg)  ((reg & BIT_STD_SET) == 0)
#endif

#if POL_TTL_SET == 1
#define VAL_TTL_SET(reg)  ((reg & BIT_TTL_SET) > 0)
#else
#define VAL_TTL_SET(reg)  ((reg & BIT_TTL_SET) == 0)
#endif


/* static data used for integration time calculations */

const struct {
  long MaxSize;
  long StepSize;
  long LineTime;
} IntegrationLookup[] =
{
  {  539L, 29L, 296L },          /* 0:  FT12, 40 MHz, 60 Hz, NI */
  { 1082L, 57L, 296L },          /* 1:  FT18, 40 MHz, 60 Hz, NI */
  {  539L, 29L, 592L },          /* 2:  FT12, 20 MHz, 60 Hz, NI */
  { 1082L, 57L, 592L },          /* 3:  FT18, 20 MHz, 60 Hz, NI */
  {  603L, 32L, 320L },          /* 4:  FT12, 40 MHz, 50 Hz, NI */
  { 1209L, 64L, 320L },          /* 5:  FT18, 40 MHz, 50 Hz, NI */
  {  603L, 32L, 640L },          /* 6:  FT12, 20 MHz, 50 Hz, NI */
  { 1209L, 64L, 640L },          /* 7:  FT18, 20 MHz, 50 Hz, NI */
  {  539L, 29L, 296L },          /* 8:  FT12, 40 MHz, 60 Hz, I */
  {  525L,  0L, 296L },          /* 9:  FT18, 40 MHz, 60 Hz, I */
  {  539L, 29L, 592L },          /* 10: FT12, 20 MHz, 60 Hz, I */
  {  525L,  0L, 592L },          /* 11: FT18, 20 MHz, 60 Hz, I */
  {  603L, 32L, 320L },          /* 12: FT12, 40 MHz, 50 Hz, I */
  {  590L,  0L, 320L },          /* 13: FT18, 40 MHz, 50 Hz, I */
  {  603L, 32L, 640L },          /* 14: FT12, 20 MHz, 50 Hz, I */
  {  590L,  0L, 640L }           /* 15: FT18, 20 MHz, 50 Hz, I */
};


/* table with gain code and value */
const struct {
  int gain;
  uchar code;
} GainLookup[] =
{
  {1,   0x1C},
  {2,   0x1D},
  {4,   0x1E},
  {8,   0x1F},
  {16,  0x03},
  {32,  0x07},
  {64,  0x0B},
  {128, 0x0F}
};


#endif  /* _MX12P_INT_LIB */







